/*

Target MCU   : ATML (AT90CAN64)
XTAL         : 16Mhz
Uart Bitrate : 115200 bps

USART 0 -> PC 

USART 1 -> NT-ARSv1 

 : 1  (0x31)
 : 2  (0x32)

*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdio.h>

#define BAUDRATE_16M_9600   103//71   //16Mhz 9600
#define BAUDRATE_16M_115200 8       //16Mhz 115200

#define FILE_OPEN	0x01
#define FILE_CLOSE	0x02

#define SET         (char)1
#define RESET       (char)0

#define USART1      (char)1
#define USART0      (char)0

#define RECIEVE     (char)0
#define CALCULATE   (char)1

#define ROLL        (char)0
#define PITCH       (char)1

static FILE* fp;
static unsigned char fp_flag = FILE_CLOSE;

volatile unsigned char gc_RecieveData[2];
volatile unsigned char gc_Flag[10];
volatile unsigned char gc_DataBuff[30];
volatile unsigned int gi_BUFCNT;




double gd_AngleData[2];

int u1_PutChar(unsigned char data)
{
    //if( data == '\n')    u1_PutChar('\r');
    loop_until_bit_is_set(UCSR1A, UDRE1);
    UDR1 = data;
    return 0;
}
int u0_PutChar(unsigned char data)
{
    //if( data == '\n')    u0_PutChar('\r');
    loop_until_bit_is_set(UCSR0A, UDRE0);
    UDR0 = data;
    return 0;
}

char u0_GetChar(void)
{
    while( !(UCSR0A & (1<<RXC0)) );		//Wait for data to be received
    return UDR0;						// Get and return received data from buffer
}
char u1_GetChar(void)
{
    while( !(UCSR1A & (1<<RXC1)) );		//Wait for data to be received
    return UDR1;						// Get and return received data from buffer
}

void u1_PutString(char *str)
{
    while(*str != '\0')
    {
        while( !(UCSR1A & (1<<UDRE1)) );	//Wait for empty transmit buffer
        UDR1 = *str;
        str++;
    }
}

void u0_PutString(char *str)
{
    while(*str != '\0')
    {
        while( !(UCSR0A & (1<<UDRE0)) );	//Wait for empty transmit buffer
        UDR0 = *str;
        str++;
    }
}


int main(void)
{

    UCSR0A = 0x00;
    UCSR0B = (1<<RXCIE0) | (1<<RXEN0)|(1<<TXEN0);
    UCSR0C = (0<<UMSEL0) | (0<<UPM0) | (0<<USBS0) | (1<<UCSZ01) | (1<<UCSZ00);

    UBRR0H = BAUDRATE_16M_115200 >> 8;
    UBRR0L = BAUDRATE_16M_115200;
    DDRE |= 0x02;
    PORTE = 0x02;

    UCSR1A = 0x00;
    UCSR1B = (1<<RXCIE1) | (1<<RXEN1)|(1<<TXEN1);
    UCSR1C = (0<<UMSEL1) | (0<<UPM1) | (0<<USBS1) | (1<<UCSZ11) | (1<<UCSZ10);
    UBRR1H = BAUDRATE_16M_115200 >> 8;
    UBRR1L = BAUDRATE_16M_115200;
    DDRD |= 0x08;
    PORTD = 0x08;

    if(fp_flag == FILE_OPEN)
    fclose(fp);
    FILE* fp;    
    fp = fdevopen(u0_PutChar, u0_GetChar);
    fp_flag = FILE_OPEN;

    _delay_ms(1000);

    sei();
    
    printf("NT-ARSv1 Example\r\n");

    while(1)
    {
        if(gc_Flag[CALCULATE] == SET)
        {
            gc_Flag[CALCULATE] = RESET;

            gd_AngleData[ROLL] = (double)(gc_DataBuff[2]-0x30)*1000;
            gd_AngleData[ROLL] += (double)(gc_DataBuff[3]-0x30)*100;
            gd_AngleData[ROLL] += (double)(gc_DataBuff[4]-0x30)*10;
            gd_AngleData[ROLL] += (double)(gc_DataBuff[5]-0x30);

            gd_AngleData[PITCH] = (double)(gc_DataBuff[7]-0x30)*1000;
            gd_AngleData[PITCH] += (double)(gc_DataBuff[8]-0x30)*100;
            gd_AngleData[PITCH] += (double)(gc_DataBuff[9]-0x30)*10;
            gd_AngleData[PITCH] += (double)(gc_DataBuff[10]-0x30);


            gd_AngleData[ROLL]  = ((gd_AngleData[ROLL]/1000.0) * 180.0 / 3.141592);
            gd_AngleData[PITCH] = ((gd_AngleData[PITCH]/1000.0) * 180.0 / 3.141592);


            if(gc_DataBuff[1] == '-')
            {
                gd_AngleData[ROLL] = -(gd_AngleData[ROLL]);
            }


            if(gc_DataBuff[6] == '-')
            {
                gd_AngleData[PITCH] = -(gd_AngleData[PITCH]);
            }

            printf("%1.2f, %1.2f\r\n",gd_AngleData[ROLL],gd_AngleData[PITCH]);
        }
    }
}

SIGNAL(SIG_UART1_RECV)              //USART 1
{
    gc_RecieveData[USART1] = UDR1;
    if(gc_RecieveData[USART1] == '<')
    {
        gi_BUFCNT = RESET;
        gc_Flag[RECIEVE] = SET;
    }
    else if(gc_RecieveData[USART1] == '>')
    {
        gc_DataBuff[gi_BUFCNT++] = gc_RecieveData[USART1];
        gc_Flag[RECIEVE] = RESET;
        gc_Flag[CALCULATE] = SET;
        gi_BUFCNT = RESET;

        if(gi_BUFCNT>29)
        {
            gi_BUFCNT = RESET;
        }
    }
    if(gc_Flag[RECIEVE] == SET)
    {
        gc_DataBuff[gi_BUFCNT++] = gc_RecieveData[USART1];
        
    }
}

SIGNAL(SIG_UART0_RECV)              //USART 0
{
    gc_RecieveData[USART0] = UDR0;
    if(gc_RecieveData[USART0] == '1')
    {
        u0_PutChar('<');
        u0_PutChar('C');
        u0_PutChar('A');
        u0_PutChar('W');
        u0_PutChar('>');

        u1_PutChar('<');
        u1_PutChar('C');
        u1_PutChar('A');
        u1_PutChar('W');
        u1_PutChar('>');
    }
    else if(gc_RecieveData[USART0] == '2')
    {
        u0_PutChar('<');
        u0_PutChar('C');
        u0_PutChar('A');
        u0_PutChar('E');
        u0_PutChar('>');

        u1_PutChar('<');
        u1_PutChar('C');
        u1_PutChar('A');
        u1_PutChar('E');
        u1_PutChar('>');
    }
    
}
